@@ -29,8 +29,8 @@
 #include <kdesktopfile.h>
 #include <ksycoca.h>
 #include <kbuildsycoca_p.h>
-#include <../src/services/kserviceutil_p.h>
-#include <../src/services/ktraderparsetree_p.h>
+#include <../src/services/kserviceutil_p.h> // for KServiceUtilPrivate
+#include <kapplicationtrader.h>
 
 #include <kservicegroup.h>
 #include <kservicetypetrader.h>
@@ -44,7 +44,6 @@
 #include <qthread.h>
 #include <qsignalspy.h>
 
-#include <QTimer>
 #include <QDebug>
 #include <QLoggingCategory>
 #include <QMimeDatabase>
@@ -73,7 +72,7 @@ void KServiceTest::initTestCase()
     setupXdgDirs();
     QStandardPaths::setTestModeEnabled(true);
 
-    QLoggingCategory::setFilterRules(QStringLiteral("kf5.kcoreaddons.kdirwatch.debug=true"));
+    QLoggingCategory::setFilterRules(QStringLiteral("*.debug=true"));
 
     // A non-C locale is necessary for some tests.
     // This locale must have the following properties:
@@ -85,7 +84,6 @@ void KServiceTest::initTestCase()
         qDebug() << "Setting locale to fr_FR.utf8 failed";
     }
 
-    m_hasKde5Konsole = false;
     eraseProfiles();
 
     if (!KSycoca::isAvailable()) {
@@ -223,6 +221,17 @@ void KServiceTest::initTestCase()
         mustUpdateKSycoca = true;
     }
 
+    // faketestapp.desktop
+    const QString testApp = QStandardPaths::writableLocation(QStandardPaths::ApplicationsLocation) + QLatin1String("/org.kde.faketestapp.desktop");
+    if (!QFile::exists(testApp)) {
+        QVERIFY(QDir().mkpath(QStandardPaths::writableLocation(QStandardPaths::ApplicationsLocation)));
+        const QString src = QFINDTESTDATA("org.kde.faketestapp.desktop");
+        QVERIFY(!src.isEmpty());
+        QVERIFY2(QFile::copy(src, testApp), qPrintable(testApp));
+        qDebug() << "Created" << testApp;
+        mustUpdateKSycoca = true;
+    }
+
     if (mustUpdateKSycoca) {
         // Update ksycoca in ~/.qttest after creating the above
         runKBuildSycoca(true);
@@ -230,6 +239,7 @@ void KServiceTest::initTestCase()
     QVERIFY(KServiceType::serviceType(QStringLiteral("FakePluginType")));
     QVERIFY(KServiceType::serviceType(QStringLiteral("FakeBasePart")));
     QVERIFY(KServiceType::serviceType(QStringLiteral("FakeDerivedPart")));
+    QVERIFY(KService::serviceByDesktopName(QStringLiteral("org.kde.faketestapp")));
 }
 
 void KServiceTest::runKBuildSycoca(bool noincremental)
@@ -248,13 +258,13 @@ void KServiceTest::cleanupTestCase()
 {
     // If I want the konqueror unit tests to work, then I better not have a non-working part
     // as the preferred part for text/plain...
-    QStringList services; services << QStringLiteral("fakeservice.desktop") << QStringLiteral("fakepart.desktop") << QStringLiteral("faketextplugin.desktop") << QStringLiteral("fakeservice_querymustrebuild.desktop");
-    Q_FOREACH (const QString &service, services) {
+    const QStringList services = QStringList() << QStringLiteral("fakeservice.desktop") << QStringLiteral("fakepart.desktop") << QStringLiteral("faketextplugin.desktop") << QStringLiteral("fakeservice_querymustrebuild.desktop");
+    for (const QString &service : services) {
         const QString fakeService = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QLatin1String("/kservices5/") + service;
         QFile::remove(fakeService);
     }
-    QStringList serviceTypes; serviceTypes << QStringLiteral("fakeplugintype.desktop");
-    Q_FOREACH (const QString &serviceType, serviceTypes) {
+    const QStringList serviceTypes = QStringList() << QStringLiteral("fakeplugintype.desktop");
+    for (const QString &serviceType : serviceTypes) {
         const QString fakeServiceType = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QLatin1String("/kservicetypes5/") + serviceType;
         //QFile::remove(fakeServiceType);
     }
@@ -304,6 +314,32 @@ void KServiceTest::testConstructorKDesktopFile() // as happens inside kbuildsyco
     QCOMPARE(KService(&desktopFile, "kservices5/fakepart.desktop").mimeTypes(), QStringList() << "text/plain" << "text/html");
 }
 
+void KServiceTest::testCopyConstructor()
+{
+    const QString fakePart = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QLatin1String("/kservices5/") + "fakepart.desktop";
+    QVERIFY(QFile::exists(fakePart));
+    KDesktopFile desktopFile(fakePart);
+    // KRun needs to make a copy of a KService that will go out of scope, let's test that here.
+    KService::Ptr service;
+    {
+        KService origService(&desktopFile);
+        service = new KService(origService);
+    }
+    QVERIFY(service->isValid());
+    QCOMPARE(service->mimeTypes(), QStringList() << "text/plain" << "text/html");
+}
+
+void KServiceTest::testCopyInvalidService()
+{
+    KService::Ptr service;
+    {
+        KService origService{QString()}; // this still sets a d_ptr, so no problem;
+        QVERIFY(!origService.isValid());
+        service = new KService(origService);
+    }
+    QVERIFY(!service->isValid());
+}
+
 void KServiceTest::testProperty()
 {
     ksycoca_ms_between_checks = 0;
@@ -341,7 +377,7 @@ void KServiceTest::testProperty()
     // Test property("X-KDE-Protocols"), which triggers the KServiceReadProperty code.
     KService::Ptr fakePart = KService::serviceByDesktopPath(QStringLiteral("fakepart.desktop"));
     QVERIFY(fakePart); // see initTestCase; it should be found.
-    QVERIFY(fakePart->propertyNames().contains(QStringLiteral("X-KDE-Protocols")));
+    QVERIFY(fakePart->propertyNames().contains(QLatin1String("X-KDE-Protocols")));
     QCOMPARE(fakePart->mimeTypes(), QStringList() << QStringLiteral("text/plain") << QStringLiteral("text/html")); // okular relies on subclasses being kept here
     const QStringList protocols = fakePart->property(QStringLiteral("X-KDE-Protocols")).toStringList();
     QCOMPARE(protocols, QStringList() << QStringLiteral("http") << QStringLiteral("ftp"));
@@ -376,6 +412,7 @@ void KServiceTest::testAllServices()
     }
     const KService::List lst = KService::allServices();
     QVERIFY(!lst.isEmpty());
+    bool foundTestApp = false;
 
     for (KService::List::ConstIterator it = lst.begin();
             it != lst.end(); ++it) {
@@ -384,7 +421,9 @@ void KServiceTest::testAllServices()
 
         const QString name = service->name();
         const QString entryPath = service->entryPath();
-        //qDebug() << name << "entryPath=" << entryPath << "menuId=" << service->menuId();
+        if (entryPath.contains("fake")) {
+            qDebug() << name << "entryPath=" << entryPath << "menuId=" << service->menuId();
+        }
         QVERIFY(!name.isEmpty());
         QVERIFY(!entryPath.isEmpty());
 
@@ -396,8 +435,8 @@ void KServiceTest::testAllServices()
             const QString menuId = service->menuId();
             if (menuId.isEmpty()) {
                 qWarning("%s has an empty menuId!", qPrintable(entryPath));
-            } else if (menuId == "org.kde.konsole.desktop") {
-                m_hasKde5Konsole = true;
+            } else if (menuId == "org.kde.faketestapp.desktop") {
+                foundTestApp = true;
             }
             QVERIFY(!menuId.isEmpty());
             lookedupService = KService::serviceByMenuId(menuId);
@@ -405,6 +444,7 @@ void KServiceTest::testAllServices()
             QCOMPARE(lookedupService->menuId(), menuId);
         }
     }
+    QVERIFY(foundTestApp);
 }
 
 // Helper method for all the trader tests
@@ -430,15 +470,11 @@ void KServiceTest::testDBUSStartupType()
     if (!KSycoca::isAvailable()) {
         QSKIP("ksycoca not available");
     }
-    if (!m_hasKde5Konsole) {
-        QSKIP("org.kde.konsole.desktop not available");
-    }
-    //KService::Ptr konsole = KService::serviceByMenuId( "org.kde.konsole.desktop" );
-    KService::Ptr konsole = KService::serviceByDesktopName(QStringLiteral("org.kde.konsole"));
-    QVERIFY(konsole);
-    QCOMPARE(konsole->menuId(), QStringLiteral("org.kde.konsole.desktop"));
-    //qDebug() << konsole->entryPath();
-    QCOMPARE(int(konsole->dbusStartupType()), int(KService::DBusUnique));
+    KService::Ptr testapp = KService::serviceByDesktopName(QStringLiteral("org.kde.faketestapp"));
+    QVERIFY(testapp);
+    QCOMPARE(testapp->menuId(), QStringLiteral("org.kde.faketestapp.desktop"));
+    //qDebug() << testapp->entryPath();
+    QCOMPARE(int(testapp->dbusStartupType()), int(KService::DBusUnique));
 }
 
 void KServiceTest::testByStorageId()
@@ -446,24 +482,15 @@ void KServiceTest::testByStorageId()
     if (!KSycoca::isAvailable()) {
         QSKIP("ksycoca not available");
     }
-    if (QStandardPaths::locate(QStandardPaths::ApplicationsLocation, QStringLiteral("org.kde.konsole.desktop")).isEmpty()) {
-        QSKIP("org.kde.konsole.desktop not available");
-    }
-    QVERIFY(KService::serviceByMenuId(QStringLiteral("org.kde.konsole.desktop")));
-    QVERIFY(!KService::serviceByMenuId(QStringLiteral("org.kde.konsole"))); // doesn't work, extension mandatory
-    QVERIFY(!KService::serviceByMenuId(QStringLiteral("konsole.desktop"))); // doesn't work, full filename mandatory
-    QVERIFY(KService::serviceByStorageId(QStringLiteral("org.kde.konsole.desktop")));
-    QVERIFY(KService::serviceByStorageId("org.kde.konsole"));
+    QVERIFY(!QStandardPaths::locate(QStandardPaths::ApplicationsLocation, QStringLiteral("org.kde.faketestapp.desktop")).isEmpty());
+    QVERIFY(KService::serviceByMenuId(QStringLiteral("org.kde.faketestapp.desktop")));
+    QVERIFY(!KService::serviceByMenuId(QStringLiteral("org.kde.faketestapp"))); // doesn't work, extension mandatory
+    QVERIFY(!KService::serviceByMenuId(QStringLiteral("faketestapp.desktop"))); // doesn't work, full filename mandatory
+    QVERIFY(KService::serviceByStorageId(QStringLiteral("org.kde.faketestapp.desktop")));
+    QVERIFY(KService::serviceByStorageId("org.kde.faketestapp"));
 
-    // This one fails here; probably because there are two such files, so this would be too
-    // ambiguous... According to the testAllServices output, the entryPaths are
-    // entryPath="/d/kde/inst/kde5/share/applications/org.kde.konsole.desktop"
-    // entryPath= "/usr/share/applications/org.kde.konsole.desktop"
-    //
-    //QVERIFY(KService::serviceByDesktopPath("org.kde.konsole.desktop"));
-
-    QVERIFY(KService::serviceByDesktopName(QStringLiteral("org.kde.konsole")));
-    QCOMPARE(KService::serviceByDesktopName(QStringLiteral("org.kde.konsole"))->menuId(), QString("org.kde.konsole.desktop"));
+    QVERIFY(KService::serviceByDesktopName(QStringLiteral("org.kde.faketestapp")));
+    QCOMPARE(KService::serviceByDesktopName(QStringLiteral("org.kde.faketestapp"))->menuId(), QString("org.kde.faketestapp.desktop"));
 }
 
 void KServiceTest::testServiceTypeTraderForReadOnlyPart()
@@ -480,7 +507,7 @@ void KServiceTest::testServiceTypeTraderForReadOnlyPart()
         || !offerListHasService(offers, QStringLiteral("fakepart2.desktop"))
         || !offerListHasService(offers, QStringLiteral("otherpart.desktop"))
         || !offerListHasService(offers, QStringLiteral("preferredpart.desktop"))) {
-        foreach (KService::Ptr service, offers) {
+        for (KService::Ptr service : qAsConst(offers)) {
             qDebug("%s %s", qPrintable(service->name()), qPrintable(service->entryPath()));
         }
     }
@@ -494,16 +521,20 @@ void KServiceTest::testServiceTypeTraderForReadOnlyPart()
 
     // Check ordering according to InitialPreference
     int lastPreference = -1;
+#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 67)
     bool lastAllowedAsDefault = true;
-    Q_FOREACH (KService::Ptr service, offers) {
+#endif
+    for (KService::Ptr service : qAsConst(offers)) {
         const QString path = service->entryPath();
         const int preference = service->initialPreference(); // ## might be wrong if we use per-servicetype preferences...
         //qDebug( "%s has preference %d, allowAsDefault=%d", qPrintable( path ), preference, service->allowAsDefault() );
+#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 67)
         if (lastAllowedAsDefault && !service->allowAsDefault()) {
             // first "not allowed as default" offer
             lastAllowedAsDefault = false;
             lastPreference = -1; // restart
         }
+#endif
         if (lastPreference != -1) {
             QVERIFY(preference <= lastPreference);
         }
@@ -576,7 +607,7 @@ void KServiceTest::testTraderConstraints()
 void KServiceTest::testSubseqConstraints()
 {
   auto test = [](const char* pattern, const char* text, bool sensitive) {
-    return KTraderParse::ParseTreeSubsequenceMATCH::isSubseq(
+    return KApplicationTrader::isSubsequence(
         QString(pattern),
         QString(text),
         sensitive? Qt::CaseSensitive : Qt::CaseInsensitive
@@ -631,6 +662,7 @@ void KServiceTest::testHasServiceType2() // with services coming from ksycoca
     QVERIFY(!faketextPlugin->hasServiceType(QStringLiteral("FakeBasePart")));
 }
 
+#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 66)
 void KServiceTest::testWriteServiceTypeProfile()
 {
     const QString serviceType = QStringLiteral("FakeBasePart");
@@ -639,10 +671,10 @@ void KServiceTest::testWriteServiceTypeProfile()
     services.append(KService::serviceByDesktopPath(QStringLiteral("fakepart.desktop")));
     disabledServices.append(KService::serviceByDesktopPath(QStringLiteral("fakepart2.desktop")));
 
-    Q_FOREACH (const KService::Ptr &serv, services) {
+    for (const KService::Ptr &serv : qAsConst(services)) {
         QVERIFY(serv);
     }
-    Q_FOREACH (const KService::Ptr &serv, disabledServices) {
+    for (const KService::Ptr &serv : qAsConst(disabledServices)) {
         QVERIFY(serv);
     }
 
@@ -665,6 +697,7 @@ void KServiceTest::testWriteServiceTypeProfile()
     QVERIFY(offerListHasService(offers, QStringLiteral("otherpart.desktop")));     // should still be somewhere in there
     QVERIFY(!offerListHasService(offers, QStringLiteral("fakepart2.desktop")));     // it got disabled above
 }
+#endif
 
 void KServiceTest::testDefaultOffers()
 {
@@ -680,6 +713,7 @@ void KServiceTest::testDefaultOffers()
     QCOMPARE(offers[0]->entryPath(), m_firstOffer);
 }
 
+#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 66)
 void KServiceTest::testDeleteServiceTypeProfile()
 {
     const QString serviceType = QStringLiteral("FakeBasePart");
@@ -694,13 +728,11 @@ void KServiceTest::testDeleteServiceTypeProfile()
     }
     QCOMPARE(offers[0]->entryPath(), m_firstOffer);
 }
+#endif
 
 void KServiceTest::testActionsAndDataStream()
 {
-    if (QStandardPaths::locate(QStandardPaths::ApplicationsLocation, QStringLiteral("org.kde.konsole.desktop")).isEmpty()) {
-        QSKIP("org.kde.konsole.desktop not available");
-    }
-    KService::Ptr service = KService::serviceByStorageId(QStringLiteral("org.kde.konsole.desktop"));
+    KService::Ptr service = KService::serviceByStorageId(QStringLiteral("org.kde.faketestapp.desktop"));
     QVERIFY(service);
     QVERIFY(!service->property(QStringLiteral("Name[fr]"), QVariant::String).isValid());
     const QList<KServiceAction> actions = service->actions();
@@ -708,9 +740,10 @@ void KServiceTest::testActionsAndDataStream()
     const KServiceAction newTabAction = actions.at(1);
     QCOMPARE(newTabAction.name(), QStringLiteral("NewTab"));
     QCOMPARE(newTabAction.exec(), QStringLiteral("konsole --new-tab"));
-    QVERIFY(newTabAction.icon().isEmpty());
+    QCOMPARE(newTabAction.icon(), QStringLiteral("tab-new"));
     QCOMPARE(newTabAction.noDisplay(), false);
     QVERIFY(!newTabAction.isSeparator());
+    QCOMPARE(newTabAction.service()->name(), service->name());
 }
 
 void KServiceTest::testServiceGroups()
@@ -727,7 +760,7 @@ void KServiceTest::testServiceGroups()
                                      true /* sort by generic name */);
 
     qDebug() << list.count();
-    Q_FOREACH (KServiceGroup::SPtr s, list) {
+    for (KServiceGroup::SPtr s : list) {
         qDebug() << s->name() << s->entryPath();
     }
 
@@ -814,7 +847,11 @@ void KServiceTest::testThreads()
     sync.addFuture(QtConcurrent::run(this, &KServiceTest::testDeletingService));
     sync.addFuture(QtConcurrent::run(this, &KServiceTest::testTraderConstraints));
     // process events (DBus, inotify...), until we get all expected signals
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
     QTRY_COMPARE_WITH_TIMEOUT(m_sycocaUpdateDone.load(), 1, 15000); // not using a bool, just to silence helgrind
+#else
+    QTRY_COMPARE_WITH_TIMEOUT(m_sycocaUpdateDone.loadRelaxed(), 1, 15000); // not using a bool, just to silence helgrind
+#endif
     qDebug() << "Joining all threads";
     sync.waitForFinished();
 }
@@ -874,6 +911,7 @@ void KServiceTest::testEntryPathToName()
     QCOMPARE(KService(QStringLiteral("/hallo/a.b.c.desktop")).name(), QStringLiteral("a.b.c"));
 }
 
+#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 0)
 void KServiceTest::testKPluginMetaData()
 {
     const QString fakePart = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QLatin1String("/kservices5/") + "fakepart.desktop";
@@ -883,6 +921,7 @@ void KServiceTest::testKPluginMetaData()
     auto info_md = info.toMetaData();
     QCOMPARE(info_md.formFactors(), md.formFactors());
 }
+#endif
 
 void KServiceTest::testTraderQueryMustRebuildSycoca()
 {
