@@ -4,7 +4,8 @@
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
-   License version 2 as published by the Free Software Foundation.
+   License as published by the Free Software Foundation; either
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,7 +29,6 @@
 #include "kmimetypefactory_p.h"
 #include "servicesdebug.h"
 #include <qmimedatabase.h>
-#include <QDebug>
 
 class KMimeTypeTraderPrivate
 {
@@ -67,7 +67,7 @@ static KServiceOfferList mimeTypeSycocaOffers(const QString &mimeType)
     QString mime = db.mimeTypeForName(mimeType).name();
     if (mime.isEmpty()) {
         if (!mimeType.startsWith(QLatin1String("x-scheme-handler/"))) { // don't warn for unknown scheme handler mimetypes
-            qWarning() << "KMimeTypeTrader: mimeType" << mimeType << "not found";
+            qCWarning(SERVICES) << "KMimeTypeTrader: mimeType" << mimeType << "not found";
             return lst; // empty
         }
         mime = mimeType;
@@ -96,7 +96,7 @@ static KService::List mimeTypeSycocaServiceOffers(const QString &mimeType)
     QString mime = db.mimeTypeForName(mimeType).name();
     if (mime.isEmpty()) {
         if (!mimeType.startsWith(QLatin1String("x-scheme-handler/"))) { // don't warn for unknown scheme handler mimetypes
-            qWarning() << "KMimeTypeTrader: mimeType" << mimeType << "not found";
+            qCWarning(SERVICES) << "KMimeTypeTrader: mimeType" << mimeType << "not found";
             return lst; // empty
         }
         mime = mimeType;
@@ -105,7 +105,7 @@ static KService::List mimeTypeSycocaServiceOffers(const QString &mimeType)
     KMimeTypeFactory *factory = KSycocaPrivate::self()->mimeTypeFactory();
     const int offset = factory->entryOffset(mime);
     if (!offset) {
-        qWarning() << "KMimeTypeTrader: mimeType" << mimeType << "not found";
+        qCWarning(SERVICES) << "KMimeTypeTrader: mimeType" << mimeType << "not found";
         return lst; // empty
     }
     const int serviceOffersOffset = factory->serviceOffersOffset(mime);
@@ -117,7 +117,7 @@ static KService::List mimeTypeSycocaServiceOffers(const QString &mimeType)
 
 #define CHECK_SERVICETYPE(genericServiceTypePtr) \
     if (!genericServiceTypePtr) { \
-        qWarning() << "KMimeTypeTrader: couldn't find service type" << genericServiceType << \
+        qCWarning(SERVICES) << "KMimeTypeTrader: couldn't find service type" << genericServiceType << \
                    "\nPlease ensure that the .desktop file for it is installed; then run kbuildsycoca5."; \
         return; \
     }
@@ -139,9 +139,7 @@ void KMimeTypeTrader::filterMimeTypeOffers(KServiceOfferList &list, const QStrin
     while (it.hasNext()) {
         const KService::Ptr servPtr = it.next().service();
         // Expand servPtr->hasServiceType( genericServiceTypePtr ) to avoid lookup each time:
-        if (!KSycocaPrivate::self()->serviceFactory()->hasOffer(genericServiceTypePtr->offset(),
-                                               genericServiceTypePtr->serviceOffersOffset(),
-                                               servPtr->offset())
+        if (!KSycocaPrivate::self()->serviceFactory()->hasOffer(genericServiceTypePtr, servPtr)
                 || !servPtr->showInCurrentDesktop()) {
             it.remove();
         }
@@ -195,10 +193,14 @@ KService::Ptr KMimeTypeTrader::preferredService(const QString &mimeType, const Q
     filterMimeTypeOffers(offers, genericServiceType);
 
     KServiceOfferList::const_iterator itOff = offers.constBegin();
+#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 67)
     // Look for the first one that is allowed as default.
     // Since the allowed-as-default are first anyway, we only have
     // to look at the first one to know.
     if (itOff != offers.constEnd() && (*itOff).allowAsDefault()) {
+#else
+    if (itOff != offers.constEnd()) {
+#endif
         return (*itOff).service();
     }
 
